<?php
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Communication\CommunicationListener;
use ManiaControl\Logger;
use ManiaControl\ManiaControl;
use ManiaControl\Maps\Map;
use ManiaControl\Maps\MapManager;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Settings\SettingManager;
use ManiaControl\Utils\Formatter;
use Maniaplanet\DedicatedServer\Xmlrpc\FileException;

/**
 * Created by PhpStorm.
 * User: XeRoX
 * Date: 10.04.2016
 * Time: 17:49
 */
class SyncParagonMappacks implements Plugin, CommunicationListener, CallbackListener, TimerListener {

	const ID      = 99;
	const VERSION = 0.5;
	const AUTHOR  = 'Jocy';
	const NAME    = 'SyncParagonMappacks';
	const DESC    = 'Snycs the maps with the pL Mappacks';

	const SETTING_GAMEMODE = 'Gamemode';

	/* @var $maniaControl ManiaControl */
	private $maniaControl = null;

	/* @var $removeMapOnNextMap Map */
	private $removeMapOnNextMap = null;

	private $gamemodes = null;

	/**
	 * Prepare the Plugin
	 *
	 * @param \ManiaControl\ManiaControl $maniaControl
	 */
	public static function prepare(ManiaControl $maniaControl) {
		// TODO: Implement prepare() method.
	}

	/**
	 * Load the plugin
	 *
	 * @param \ManiaControl\ManiaControl $maniaControl
	 * @return bool
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;

		$url = 'https://league.paragon-esports.com/ws/getActiveGamemodes';

		$this->maniaControl->getFileReader()->loadFile($url, function ($data, $error) {
			$this->gamemodes = json_decode($data);

			$array     = get_object_vars($this->gamemodes);
			$gamemodes = array_keys($array);

			$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_GAMEMODE, $gamemodes);

			//In closure because it needs the gamemode
			$this->checkMaps();
		});

		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERDISCONNECT, $this, 'handlePlayerDisconnect');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::BEGINMAP, $this, function () {
			// Check if the previous played map should be removed
			if ($this->removeMapOnNextMap) {
				$this->maniaControl->getMapManager()->removeMap(null, $this->removeMapOnNextMap->uid);
				$this->removeMapOnNextMap = null;
			}

		});
	}

	/**
	 * Unload the plugin and its Resources
	 */
	public function unload() {
		// TODO: Implement unload() method.
	}

	public function handlePlayerDisconnect() {
		if (count($this->maniaControl->getPlayerManager()->getPlayers()) == 0) {
			$this->checkMaps();
		}
	}


	private function checkMaps() {

		$gamemode = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_GAMEMODE);

		$url = 'https://league.paragon-esports.com/ws/getMaps/' . $gamemode;

		if (!$url) {
			$this->maniaControl->getChat()->sendError('Please specify an url in the syncMaps plugin');
			return;
		}

		if(!$this->gamemodes){
			$this->maniaControl->getChat()->sendError('Could not get gamemode list from the server');
		}

		if($this->maniaControl->getServer()->titleId != $this->gamemodes->$gamemode){
			$this->maniaControl->getChat()->sendError('The server is not in the correct gamemode for this mappack');
			return;
		}

		$this->maniaControl->getFileReader()->loadFile($url, function ($data, $error) {
			$maps = json_decode($data);

			if (!$maps) {
				$this->maniaControl->getChat()->sendError('Could not get mapslist from the server');
				return;
			}

			$backedUp = false;

			foreach ($maps as $map) {
				$serverMap = $this->maniaControl->getMapManager()->getMapByMxId($map->mx_id);
				if (!$serverMap) {
					$this->maniaControl->getMapManager()->addMapFromMx($map->mx_id, null);
					if (!$backedUp) {
						$this->backupMatchSettings();
						$backedUp = true;
					}

				} else {
					if ($serverMap->updateAvailable()) {
						$this->maniaControl->getMapManager()->updateMap(null, $serverMap->uid);
					}
				}
			}

			//Fix if it would remove every map from the server (slow mx)
			$this->maniaControl->getTimerManager()->registerOneTimeListening($this, function () use ($maps, $backedUp) {
				$serverMaps = $this->maniaControl->getMapManager()->getMaps();

				foreach ($serverMaps as $serverMap) {
					$shouldBeOnServer = false;
					foreach ($maps as $map) {
						if ($serverMap->mx && $serverMap->mx->id == $map->mx_id) {
							$shouldBeOnServer = true;
						}
					}
					if (!$shouldBeOnServer) {
						//var_dump()
						//exit();
						if ($this->maniaControl->getMapManager()->getCurrentMap()->uid != $serverMap->uid) {
							$this->maniaControl->getMapManager()->removeMap(null, $serverMap->uid);
							if (!$backedUp) {
								$this->backupMatchSettings();
								$backedUp = true;
							}

						} else {
							// Let the current map be removed on the start of the next map
							$this->removeMapOnNextMap = $serverMap;
						}
					}
				}
			}, 15000);
		});

	}

	//Fehlt nurnoch die Serverseite

	private function backupMatchSettings() {
		$erverlogin            = $this->maniaControl->getServer()->login;
		$date                  = date('Y-m-d', time());
		$matchSettingsFileName = "MatchSettings/Backups/{$erverlogin}-{$date}.txt";
		try {
			$this->maniaControl->getClient()->saveMatchSettings($matchSettingsFileName);
		} catch (FileException $e) {
			Logger::logError("Unable to write the playlist file, please checkout your MX-Folders File permissions!");
		}
	}

	/**
	 * Get plugin id
	 *
	 * @return int
	 */
	public static function getId() {
		return self::ID;
	}

	/**
	 * Get Plugin Name
	 *
	 * @return string
	 */
	public static function getName() {
		return self::NAME;
	}

	/**
	 * Get Plugin Version
	 *
	 * @return string
	 */
	public static function getVersion() {
		return self::VERSION;
	}

	/**
	 * Get Plugin Author
	 *
	 * @return string
	 */
	public static function getAuthor() {
		return self::AUTHOR;
	}

	/**
	 * Get Plugin Description
	 *
	 * @return string
	 */
	public static function getDescription() {
		return self::DESC;
	}
}